#
# MailWatch for MailScanner
# Copyright (C) 2003  Steve Freegard (smf@f2s.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use DBI;
use Sys::Hostname;

# Trace settings - uncomment this to debug
# DBI->trace(2,'/root/dbitrace.log');

my($dbh);
my($sth);
my($hostname) = hostname;

# Modify this as necessary for your configuration
my($db_name) = "mailscanner";
my($db_host) = "localhost";
my($db_user) = "root";
my($db_pass) = "";

sub InitMailWatchLogging {
 # Don't create another connection if one already exists
 return if defined $dbh;

 MailScanner::Log::InfoLog("Initialising database connection");

 # Connect to the database
 $dbh = DBI->connect("DBI:mysql:database=$db_name;host=$db_host",
                     $db_user, $db_pass,
                     {PrintError => 0}); 
 
 # Check if connection was successfull - if it isn't
 # then generate a warning and continue processing.
 if (!$dbh) {
  MailScanner::Log::WarnLog("Unable to initialise database connection: %s", $DBI::errstr);
  return;
 } else {
  # Prepare statement
  $sth = $dbh->prepare("INSERT DELAYED INTO maillog VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)") or MailScanner::Log::WarnLog($DBI::errstr);
 }

 MailScanner::Log::InfoLog("Finished initialising database connection");
}

sub EndMailWatchLogging {
 $dbh->disconnect if defined $dbh;
 MailScanner::Log::InfoLog("Disconnected from the database");
}

sub MailWatchLogging {
  my($message) = @_;

  # Don't bother trying to do an insert if  no message is passed-in
  return unless $message;
  
  # Check to make sure DB connection is still valid
  if(!$dbh->ping) {
   MailScanner::Log::InfoLog("Database ping failure attempting to re-connect");
   InitMailWatchLogging();
  }
 
  # Stop processing here if the connection to the database is down
  return unless defined $dbh;

  # Get rid of control chars and tidy-up SpamAssassin report
  my $spamreport = $message->{spamreport};
  $spamreport =~ s/\n/ /g;
  $spamreport =~ s/\t//g;

  # Get timestamp, and format it so it is suitable to use with MySQL
  my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
  my($timestamp) = sprintf("%d-%02d-%02d %02d:%02d:%02d",
			   $year+1900,$mon+1,$mday,$hour,$min,$sec);
  
  my($date) = sprintf("%d-%02d-%02d",$year+1900,$mon+1,$mday);
  my($time) = sprintf("%02d:%02d:%02d",$hour,$min,$sec);

  # Also print 1 line for each report about this message. These lines
  # contain all the info above, + the attachment filename and text of
  # each report.
  my($file, $text, @report_array);
  while(($file, $text) = each %{$message->{allreports}}) {
    $file = "the entire message" if $file eq "";
    # Use the sanitised filename to avoid problems caused by people forcing
    # logging of attachment filenames which contain nasty SQL instructions.
    $file = $message->{file2safefile}{$file} or $file;
    $text =~ s/\n/ /;  # Make sure text report only contains 1 line
    $text =~ s/\t/ /; # and no tab characters
    push (@report_array, $text);
  }

  # Sanitize reports
  my $reports = join(",",@report_array);

  # Insert the data
  $sth->execute(
   $timestamp,
   $message->{id},
   $message->{size},
   $message->{from},
   join(',', @{$message->{to}}), 
   $message->{subject},
   $message->{clientip},
   join(',', @{$message->{archiveplaces}}),
   $message->{isspam},
   $message->{ishigh},
   $message->{issaspam},
   $message->{isrblspam},
   $message->{spamwhitelisted},
   $message->{sascore},
   $spamreport,
   $message->{virusinfected},
   $message->{nameinfected},
   $message->{otherinfected},
   $reports,
   $hostname,
   $date,
   $time,
   join('\n',@{$message->{headers}}))
 or MailScanner::Log::WarnLog("Cannot insert row: %s", $sth->errstr);
}

1;
